%
% This script plots sigma-0 versus azimuth angle for each of QuikSCAT's
% (upto) 4 measurements to illustrate how wind retrieval works
%

% written by D.Long at BYU
% last revised 20 Jul 2019 by D.Long at BYU

% load GMF
load_QMOD4;

fprintf('BYU QMOD4plt\n');

% example sweet spot QuikSCAT values
def_Nmeas=4;
def_inc=[46 46 54 54];
def_azi=[208 308 198 318];

% loop forever
while 1

  % Enter "true wind"
  wspd=input('\nEnter true wind speed: (m/s 10m neutral stability) ');
  if length(wspd)==0 | Nmeas==0, return; end;
  wdir=input('Enter true wind direction: (deg rel to north, oceanographic conven) ');
  if length(wdir)==0, wdir=0; end;
  
  % Enter geometry information
  Nmeas=input('Enter number of sigma-0 measurements at wvc (1..4): ');
  if length(Nmeas)==0, Nmeas=def_Nmeas; end;
  
  % For each measurement, request geometry
  for k=1:Nmeas
    inc_in=input(sprintf('Enter measurement %d of %d incidence angle: (40-60 deg) ',k,Nmeas));
    if length(inc_in)==0 
      inc(k)=def_inc(k);
    else
      inc(k)=inc_in;
    end;
    azi_in=input(sprintf('Enter measurement %d of %d radar azimuth angle: (0-360 deg) ',k,Nmeas));
    if length(azi_in)==0
      azi(k)=def_azi(k);
    else
      azi(k)=azi_in;
    end;
    
    % compute true sigma-0 based on GMF
    sig(k)=interp_QMOD4(azi(k),inc(k),wspd,wdir);    
  end
  
  % summarize inputs and noise-free sigma-0 measurements
  fprintf('\nTrue wind: %6.2f m/s %7.2f deg\n',wspd,wdir);
  for k=1:Nmeas
    fprintf(' Meas %d of %d: inc=%6.2f  az=%7.2f  sig=%7.2f dB  sig=%f\n',k,Nmeas,inc(k),azi(k),10*log10(sig(k)),sig(k));
  end

  % now generate plots of possible solutions, which are directions and
  % speeds which give rise to same sigma-0 for each sigma-0 measurement
  % geometry. Compute a simplified wind retrieval objective function
  % and show ambiguities which are the local mins of the objective
  % function. Note: actual sigma-0 measurements are noisy so error is
  % never zero as occurs in this graphical example.  
  
  myfigure(1)
  plot(wdir,wspd,'k*');  
  hold on;
  alab={};
  alab{1}='True';
  serr=0;
  for k=1:Nmeas
    [d,s(k,:)]=lookup_sigma0_QMOD4(azi(k),inc(k),sig(k));
    serr=serr+(s(k,:)-wspd).^2;
    alab{k+1}=sprintf('Meas %d',k);
    if k<Nmeas
      plot(d,s(k,:));
    else
      [a,h1,h2]=plotyy(d,s(k,:),d,serr);
      set(h2,'LineStyle','--','LineWidth',1);
      %alab{k+3}='RMSerr';
    end
  end
  plot(d,mean(s,1),'c:')
  alab{k+2}='mspd';
  hold off;
  %legend(alab)
  set(a(1),'Xlim',[0 360]);
  set(a(2),'Xlim',[0 360]);
  xlabel('Wind direction (deg)');
  ylabel('Wind speed (m/s)');
  set(get(a(2),'YLabel'),'String','RMSErr');

  % informative title
  str=sprintf('QMOD4 GMF   True wind: %6.2f m/s %7.2f deg\n',wspd,wdir);
  for k=1:Nmeas
    str=[str sprintf(' Meas %d of %d: inc=%5.1f  az=%6.1f  sig=%7.2f dB  sig=%f',k,Nmeas,inc(k),azi(k),10*log10(sig(k)),sig(k))];
    if k<Nmeas, str=[str sprintf('\n')]; end;
  end
  h=title(str);
  set(h,'FontWeight','normal') 

  % show ambiguities based on speed err using build-in matlab routine
  % (because of numerical approximations, it helps to smooth function
  %  but this must be done cyclically)
  %
  % Note: this is *NOT* how the ambiguities are actually computed. this
  % is just for illustration purposes.
  
  objf=smooth([serr(end-1) serr(end) serr serr(1) serr(2)]);
  [mins,locs]=findpeaks(-objf);
  mins=-mins;
  locs=locs-2;
  locs(locs<1)=locs(locs<1)+360;
  locs(locs>360)=locs(locs>360)-360;
  
  % special case because illustrative algorithm is stupid
  if locs(end)==360 && locs(1)==1, locs=locs(1:end-1); end;
  if locs(1)==360 && locs(end)==1, locs=locs(2:end); end;
  
  Namb=length(locs);
  ambdirs=d(locs);
  for k=1:Namb
    ambspds(k)=mean(s(:,locs(k)));
  end
  hold on;
  for k=1:Namb
    plot(ambdirs(k),ambspds(k),'go','LineWidth',1.5)
  end
  hold off
  alab{Nmeas+3}='Aliases';
  legend(alab)
  
  % summarize ambiguities
  fprintf('Number of Ambiguities: %d\n',Namb);
  for k=1:Namb
    fprintf(' Ambiguity %d of %d: dir=%7.2f  spd=%6.2f  ObjF=%f\n',k,Nmeas,ambdirs(k),ambspds(k),-serr(locs(k)));
  end
  
end